# Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

"""
:mod:`gfm.strikethrough` -- Strike-through support
==================================================

The :mod:`gfm.strikethrough` module provides GitHub-like syntax for
strike-through text, that is text between double tildes:
``some ~~strike-through'ed~~ text``

Typical usage
-------------

.. testcode::

   import markdown
   from gfm import StrikethroughExtension

   print(markdown.markdown("I ~~like~~ love you!",
                           extensions=[StrikethroughExtension()]))

.. testoutput::

   <p>I <del>like</del> love you!</p>

"""

import markdown.inlinepatterns

STRIKE_RE = r"(~{2})(.+?)(~{2})"  # ~~strike~~


class StrikethroughExtension(markdown.Extension):
    """
    An extension that adds support for strike-through text between two ``~~``.
    """

    def extendMarkdown(self, md):
        md.inlinePatterns.register(
            markdown.inlinepatterns.SimpleTagPattern(STRIKE_RE, "del"),
            "gfm-strikethrough",
            100,
        )



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import markdown

from markdown.extensions.fenced_code import FencedCodeExtension, FencedBlockPreprocessor


class StandaloneFencedCodeExtension(FencedCodeExtension):
    def __init__(self, **kwargs):
        self.config = {
            "linenums": [False, "Use lines numbers. True=yes, False=no, None=auto"],
            "guess_lang": [False, "Automatic language detection - Default: True"],
            "css_class": [
                "highlight",
                "Set class name for wrapper <div> - " "Default: codehilite",
            ],
            "pygments_style": [
                "default",
                "Pygments HTML Formatter Style " "(Colorscheme) - Default: default",
            ],
            "noclasses": [
                False,
                "Use inline styles instead of CSS classes - " "Default false",
            ],
            "use_pygments": [
                True,
                "Use Pygments to Highlight code blocks. "
                "Disable if using a JavaScript library. "
                "Default: True",
            ],
        }
        # Markdown 3.3 introduced a breaking change.
        if markdown.__version_info__ >= (3, 3):
            super().setConfigs(kwargs)
        else:
            super().__init__(**kwargs)

    def extendMarkdown(self, md):
        """ Add FencedBlockPreprocessor to the Markdown instance. """
        md.registerExtension(self)
        # Markdown 3.3 introduced a breaking change.
        if markdown.__version_info__ >= (3, 3):
            processor = FencedBlockPreprocessor(md, self.config)
            processor.codehilite_conf = self.getConfigs()
        else:
            processor = FencedBlockPreprocessor(md)
            processor.checked_for_codehilite = True
            processor.codehilite_conf = self.config
        md.preprocessors.register(processor, "fenced_code_block", 25)



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

"""
:mod:`gfm` -- Base module for GitHub-Flavored Markdown
======================================================
"""

from gfm import autolink
from gfm import automail
from gfm import standalone_fenced_code
from gfm import semi_sane_lists
from gfm import strikethrough
from gfm import tasklist

AutolinkExtension = autolink.AutolinkExtension
AutomailExtension = automail.AutomailExtension
StandaloneFencedCodeExtension = standalone_fenced_code.StandaloneFencedCodeExtension
SemiSaneListExtension = semi_sane_lists.SemiSaneListExtension
StrikethroughExtension = strikethrough.StrikethroughExtension
TaskListExtension = tasklist.TaskListExtension

__all__ = [
    "AutolinkExtension",
    "AutomailExtension",
    "SemiSaneListExtension",
    "StandaloneFencedCodeExtension",
    "StrikethroughExtension",
    "TaskListExtension",
]



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

"""
:mod:`gfm.semi_sane_lists` -- GitHub-like list parsing
======================================================

The :mod:`gfm.semi_sane_lists` module provides an extension that causes lists
to be treated the same way GitHub does.

Like the ``sane_lists`` extension, GitHub considers a list to end if it's
separated by multiple newlines from another list of a different type. Unlike
the ``sane_lists`` extension, GitHub will mix list types if they're not
separated by multiple newlines.

GitHub also recognizes lists that start in the middle of a paragraph. This is
currently not supported by this extension, since the Python parser has a
deeply-ingrained belief that blocks are always separated by multiple newlines.

Typical usage
-------------

.. testcode::

   import markdown
   from gfm import SemiSaneListExtension

   print(markdown.markdown(\"""
   - eggs
   - milk

   1. mix
   2. stew
   \""", extensions=[SemiSaneListExtension()]))

.. testoutput::

   <ul>
   <li>eggs</li>
   <li>milk</li>
   </ul>
   <ol>
   <li>mix</li>
   <li>stew</li>
   </ol>

"""

import markdown


class SemiSaneOListProcessor(markdown.blockprocessors.OListProcessor):
    SIBLING_TAGS = ["ol"]


class SemiSaneUListProcessor(markdown.blockprocessors.UListProcessor):
    SIBLING_TAGS = ["ul"]


class SemiSaneListExtension(markdown.Extension):
    """
    An extension that causes lists to be treated the same way GitHub does.
    """

    def extendMarkdown(self, md):
        md.parser.blockprocessors.register(
            SemiSaneOListProcessor(md.parser), "olist", 41
        )
        md.parser.blockprocessors.register(
            SemiSaneUListProcessor(md.parser), "ulist", 31
        )
